package com.triennium.sonic.xmlvalidation;

import java.util.concurrent.Callable;

import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.helpers.DefaultHandler;

import com.sonicsw.xq.XQLog;

public class SAXErrorHandler extends DefaultHandler {
    private XQLog logger = null;
    private Callable callBack = null;
    private StringBuffer errors = null;
    private boolean debug = false;
    
    
    /** Creates a new instance of SAXErrorLogger.
     */
    public SAXErrorHandler(XQLog log, Callable callBack, boolean debug) throws Exception {
    	this.logger = log;
    	this.callBack=callBack;
    	this.errors = new StringBuffer();
    	this.debug=debug;	
    }
    
    
    /** Reports in the log file, but does not consume the exception.
     */
    public void error(SAXParseException exception) throws SAXException {    	        
       // throw exception;
        if (debug) logger.logError("Error occured during SAXParsing: " + exception.getMessage());
        errors.append(exception.getMessage()+"\n");
        try { callBack.call(); } catch (Exception e) { e.printStackTrace(); } //throw new SAXException(e); }
    }
    
    /** Reports in the log file, but does not consume the exception.
     */
    public void fatalError(SAXParseException exception) throws SAXException {
      //  throw exception;
    	if (debug) logger.logError("Fatal error occured during SAXParsing: " + exception.getMessage());
        errors.append(exception.getMessage()+"\n");
        try { callBack.call(); } catch (Exception e) { e.printStackTrace(); } //throw new SAXException(e); }
    }
    
    /** Consumes the exception, but reports in the log file.
     */
    public void warning(SAXParseException exception) throws SAXException {
    	if (debug) logger.logWarning("Warning during SAXParsing: " + exception.getMessage());
        errors.append(exception.getMessage()+"\n");
    }
    
    /** Logs a success entry to the logger.
     */
    public void endDocument() {
      //  logger.logInformation("document finished successfuly");
    }
    
    public String toString() {
    	return errors.toString();
    }
}
