package com.triennium.sonic.xmlvalidation;

import java.io.IOException;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.InputSource;
import org.xml.sax.SAXException;


/** An external xml validator based on the SAX parser.
 *
 */
public class SAXValidator extends AbstractValidator {
    
    private SAXParserFactory spf = null;
    private boolean errorsFound = false;
    
    /** Creates a new instance of SAXValidator */
    public SAXValidator() {
        spf = SAXParserFactory.newInstance();        
        spf.setNamespaceAware(true);
        spf.setValidating(true);
        this.errorsFound = false;
    }
    
    /** validate(java.io.File xmlFile)
     *  function validate given XML file 
     *  @param xmlFile - file for validating*/
    public boolean validate(java.io.File xmlFile) throws Exception {
        try {
            SAXParser parser = spf.newSAXParser();
            parser.setProperty(JAXP_SCHEMA_LANGUAGE,W3C_XML_SCHEMA);            
            parser.setProperty(JAXP_SCHEMA_SOURCE, this.schema);
            parser.parse(xmlFile, errorHandler); 
            
            if (errorsFound) throw new ValidationException("Errors found during validation:" + errorHandler.toString());
            
            return (!errorsFound);
        }
        catch (SAXException sex) {
            throw sex;
            //return false;
        }
    }
    
    public Object call() {
    	errorsFound = true;
    	return null;
    }
    
    /** validate(InputSource is) 
     *  function validate given XML file 
     *  @param is - InputSource for validating*/
    public boolean validate(InputSource is) throws ValidationException{
       try {
            SAXParser parser = spf.newSAXParser();
            parser.setProperty(JAXP_SCHEMA_LANGUAGE,W3C_XML_SCHEMA);
            parser.setProperty(JAXP_SCHEMA_SOURCE, this.schema);
            parser.parse(is, errorHandler);        
            
          
            if (errorsFound) throw new ValidationException("Errors found during validation: " + errorHandler.toString());

            return (!errorsFound);
       } catch (SAXException sex) {
        	throw new ValidationException(sex);
        } catch (IOException ie) {
        	throw new ValidationException(ie);
        } catch (ParserConfigurationException pce) {
        	throw new ValidationException (pce);
        }
    }
    
}

