 

package com.triennium.sonic.xmlvalidation;

import java.io.IOException;
import java.io.StringReader;

import javax.xml.parsers.ParserConfigurationException;

import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import com.sonicsw.xq.XQConstants;
import com.sonicsw.xq.XQEnvelope;
import com.sonicsw.xq.XQInitContext;
import com.sonicsw.xq.XQLog;
import com.sonicsw.xq.XQMessage;
import com.sonicsw.xq.XQMessageException;
import com.sonicsw.xq.XQParameters;
import com.sonicsw.xq.XQService;
import com.sonicsw.xq.XQServiceContext;
import com.sonicsw.xq.XQServiceException;

public class ValidationServiceType implements XQService {

	private AbstractValidator validator = null;
	private XQLog log;
	private XQParameters params;
	private String schemaFile = "";
	private boolean debug = false;

	public ValidationServiceType() {
		// Do nothing.
	}

	public void init(XQInitContext initialContext)	throws XQServiceException {

		params = initialContext.getParameters();
		log = initialContext.getLog();
		schemaFile = params.getInfo("schema", XQConstants.PARAM_STRING).getRef();
	
		if (params.getInfo("debug", XQConstants.PARAM_STRING).getValue()!=null && !params.getInfo("debug", XQConstants.PARAM_STRING).getValue().equals("")) {
			if (params.getInfo("debug", XQConstants.PARAM_STRING).getValue().equalsIgnoreCase("false") || params.getInfo("debug", XQConstants.PARAM_STRING).getValue().equalsIgnoreCase("yes")) {
				debug = false;
			}
		}
	}

	public void destroy() {
			//
	}

	public void service(XQServiceContext ctx) throws XQServiceException {
		
		validator = new SAXValidator();
	
		// Get the runtime property for the schemaFile
		final String runtimeSchemaFile = ctx.getParameters().getInfo("schema", XQConstants.PARAM_STRING).getRef();
		if (runtimeSchemaFile!=null && runtimeSchemaFile.length()>0) {
			schemaFile = runtimeSchemaFile;
		} else {
			schemaFile = "";
		}
		
		final String runtimeDebugParam = ctx.getParameters().getInfo("debug", XQConstants.PARAM_STRING).getValue();
		if (runtimeDebugParam!=null && runtimeDebugParam.length()>0) {
			if (runtimeDebugParam.equalsIgnoreCase("true") || runtimeDebugParam.equalsIgnoreCase("yes")) {
				debug = true; 
			} else {
				debug = false;
			}
		}
		
		// Get the stepname
		final String stepName = ctx.getProcessContext().getStepName();
		
		if (ctx == null)
			throw new XQServiceException("Service Context cannot be null.");
		else {

			XQEnvelope env = null;
			XQEnvelope faultEnv = null;
			boolean validationOutcome = false;

			while (ctx.hasNextIncoming()) {
				env = ctx.getNextIncoming();

				if (env != null) {
					XQMessage msg = env.getMessage();
				
					try {
						// Check whether to use initParam of runtimeParam for SchemaLocation
						if (schemaFile.equals("")) {
							log.logInformation("Using JMSHeader for schemaFile");
							if (msg.getStringHeader("schemaFile")!=null) {
								schemaFile = msg.getStringHeader("schemaFile");
							} else {
								log.logInformation("Error: no schemaFile could be found!");
								schemaFile = "";
							}
						}
						
						
						String mID = msg.getHeaderValue("JMSMessageID").toString();
						msg.setHeaderValue("MessageID", mID);
						
						if (debug) log.logInformation("Type of the message : "	+ msg.getPart(0).getContentType());

						// In case of debugging one might use a SAXErrorHandler
						try {						 
							DefaultHandler saxErrorHandler = new SAXErrorHandler(log, validator, debug);
							//if (debug) {
							validator.setErrorHandler(saxErrorHandler);
						//	} else {
							//	validator.setErrorHandler(null);
						//	}
							// Set the schema to the parser
							validator.setSchema(schemaFile);
							
							final String content = msg.getPart(0).getContent().toString();
							
							// Validate!
							validationOutcome = ((SAXValidator)validator).validate(new InputSource(new StringReader(content)));
				
						} catch (Exception e) {
							
							// Printstacktrace in debug mode.
							if (debug) {
								e.printStackTrace();
							}

							validationOutcome = false;
							String reason = null;
						
							
							/**
							 * This next bit is to decide wether the error was
							 * Avoidable --> User mistake, therefore uneligible for resubmittance --> RME Endpoint
							 * Unavoidable --> System mistake, therefore eligible for resubmittance --> Fault Endpoint
							 */
							
							// Send message to RME queue and adding Exception
							
							XQMessage errorMessage = ctx.getMessageFactory().createMessage();
							for (int p=0;p<msg.getPartCount();p++){
								errorMessage.addPart(msg.getPart(p));
							}
							
							//String content = errorMessage.getPart(0).getContent().toString();
							final String contentType = errorMessage.getPart(0).getContentType();
							final String exception = "-----------------------------------------------------\nException caught: " + e.toString();
						
							errorMessage.addPart(errorMessage.createPart(exception, contentType));						
							faultEnv = ctx.getEnvelopeFactory().createDefaultEnvelope();
							faultEnv.clearAddresses();
							faultEnv.setMessage(errorMessage);
							
							
							
							if (e instanceof ValidationException) {
								ValidationException v = (ValidationException) e;
								if (v.getCause() instanceof SAXException) {
									reason = "SAXException: " + v.getMessage();								
									faultEnv.addAddress(ctx.getProcessContext().getRMEAddress());
								}
								else if (v.getCause() instanceof IOException) {
									reason = "IOException: " + v.getMessage();
									faultEnv.addAddress(ctx.getProcessContext().getFaultAddress());
								} 
								else if (v.getCause() instanceof ParserConfigurationException) {
									reason = "ParserConfigurationException: " + v.getMessage();
									faultEnv.addAddress(ctx.getProcessContext().getFaultAddress());
								} else {
									reason = "ValidationException: " + v.getMessage();
									faultEnv.addAddress(ctx.getProcessContext().getRMEAddress());
								}
							}
							
							log.logError("VALIDATION ERROR message : " + mID+ " reason : " + reason);			
							
						}

						// Pass message onto the outbox.
						
	
						//if (!addressList.hasNext()) {
							// No forwarding.
						//	if (debug) log.logInformation("No forwarding");
						//} else {
							// Add the message to the Outbox, then record the destination addresses
							if (validationOutcome) {
	
								if (debug) log.logInformation("Success: Message is valid");
								ctx.addOutgoing(env);
								
							} else {
								if (debug) log.logInformation("Failure: Message is invalid");
								msg.setStringHeader("stepName", stepName);
//								if (faultEnv==null) {
//									faultEnv = ctx.getEnvelopeFactory().createDefaultEnvelope(msg);
//								}
								ctx.addFault(faultEnv);
								
							}
						//} 
					} catch (XQMessageException me) {
						/**
						 * In case of an unavoidable exception
						 */
							log.logError("Exception accessing XQMessage : " + me.getMessage());
							faultEnv = ctx.getEnvelopeFactory().createDefaultEnvelope(msg);
							faultEnv.addAddress(ctx.getProcessContext().getFaultAddress());
							ctx.addFault(faultEnv);		
					}
				}
			}
		}
	}

}